﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.XR.Interaction.Toolkit;

/// <summary>
/// Socket interactor, which handles the hover materials and connection of part
/// </summary>
[RequireComponent(typeof(SocketMaterial))]
public class Socket : XRSocketInteractor
{
    // Parts that are compatible with this socket
    public PartType[] types;

    private SocketMaterial socketMaterial;
    private bool isConnected = false;
    public bool IsConnected
    {
        get{ return isConnected; }
        set{ isConnected = value; }
    }
    protected override void Awake()
    {
        base.Awake();
        socketMaterial = GetComponent<SocketMaterial>();
    }
    /// <summary>
    /// Called upon hovering object into this socket class
    /// </summary>
    protected override void OnHoverEntered(HoverEnterEventArgs args)
    {
        base.OnHoverEntered(args);
        socketMaterial.MaterialHover();
    }
    /// <summary>
    /// Called upon unhovering object from this socket class
    /// </summary>
    protected override void OnHoverExited(HoverExitEventArgs args)
    {
        base.OnHoverExited(args);
        socketMaterial.MaterialDefault();
    }
    /// <summary>
    /// Called upon selecting object into this socket class
    /// </summary>
    protected override void OnSelectEntered(SelectEnterEventArgs args)
    {
        base.OnSelectEntered(args);
        socketMaterial.MaterialOff();
        
        // Double check the compatibility
        IsConnected = Connected(args.interactable);
    }
    /// <summary>
    /// Called upon unselecting object from this socket class
    /// </summary>
    protected override void OnSelectExited(SelectExitEventArgs args)
    {
        base.OnSelectExited(args);
        socketMaterial.MaterialOn();
        
        IsConnected = false;
    }
    /// <summary>
    /// Decides, whether the interactable can be selected into this socket
    /// </summary>
    public override bool CanSelect(XRBaseInteractable interactable)
    {
        return base.CanSelect(interactable) && Connected(interactable);
    }
    /// <summary>
    /// Decides, whether the interactable can be hovered into this socket
    /// </summary>
    public override bool CanHover(XRBaseInteractable interactable)
    {
        return base.CanHover(interactable) && Connected(interactable);
    }
    /// <summary>
    /// Check if this interactable is connected correctly
    /// </summary>
    public bool Connected(XRBaseInteractable interactable)
    {
        if(interactable is Part part)
        {
            foreach(PartType type in types)
            {
                if(type != part.type)
                    return false;
            }
            return true;
        }
        return false;
    }
}
